#!/usr/bin/perl

##satoriMailReciver.pl
##
##添付ファイルをメールから取り出し、ファイルシステムに保存、Gallery2DBに登録、
##記事DBの記事に、画像へのリンクを付加する。
##
##ホームディレクトリの.forwardに|pipe <satoriMailReciver.pl>と記述すれば、
##メール受信ごとにこのプログラムが起動し、標準入力にメールが渡される。
##
##エラー発生時は実行ディレクトリにMailDecode.logというファイルが作られ、その中にエラーログが書かれる。
##
##shinora@crew.sfc.keio.ac.jp
##
##070129 コメント等微修正
##070115 ファイル分け
##0612** 作成
##


use File::Basename;
use MIME::Base64;
use MIME::Parser;
use MIME::WordDecoder;
use Encode;
use DBI;
use utf8;

require "./setting.pl";
require "./getDataFromMail.pl";

eval{
  &main(shift);
};
if($@){
  &writeError($@);
}

sub main {

  my $input = \*STDIN;
  local $galleryDB;
  local $contentDB;
  
  eval{
    #メールから添付ファイル、書き込みファイル名、対象記事IDを読みこむ
    my %data = getDataFromMail($input);

    #DBの準備
    $galleryDB = &prepareDB($GALLERY_DATABASE);
    $contentDB = &prepareDB($JOOMLA_DATABASE);

    #タグつきの本文を作成する
    my $imageId = &getImageId($galleryDB);#DBから、イメージを登録した場合のIDを取得
    my $fulltext = &createContentText($contentDB,$data{'contentId'},$imageId);#タグつきの本文を作成

    #サーバを更新する
    &updateFileSystem($data{'fileName'},$data{'file'});
    &updateGalleryDB($galleryDB, $imageId, $data{'fileName'}, $data{'title'}, length($data{'file'}));
    &updateContentDB($contentDB,$data{'contentId'},$fulltext);
  };
  if($@){
    #エラーが起きたのでロールバックする
    if(defined $galleryDB){
      $galleryDB->rollback;
      $galleryDB->disconnect();
    }
    
    if(defined $contentDB){
      $contentDB->rollback;
      $contentDB->disconnect();
    }
    die $@;
  }
  #コミットする
  $galleryDB->commit();
  $galleryDB->disconnect();

  $contentDB->commit();
  $contentDB->disconnect();
  return;
}


#ファイルを書き出す
sub updateFileSystem{
  my $fileName = shift;
  my $file = shift;
  
  #引数のチェック
  if((!defined $fileName) or (!defined $file)){
    die "writeFileSystem Invalid argument";
  }

  #ファイルを書き込む
  open(FILE, "> $OUTPUT_PATH/$fileName") or
    die "file can't create, fileName = $OUTPUT_PATH/$fileName";
  binmode(FILE);
  print FILE $file;

  close(FILE);
  return $fileName;
}

#Gallery2データベースに画像を登録する
sub updateGalleryDB{
  my $db = shift;
  my $id = shift;
  my $filename = shift;
  my $title = shift;
  my $fileLength = shift;

  #引数のチェック
  if((!defined $db) or(!defined $id) or (!defined $filename) or (!defined $title) or (!defined $fileLength)){
    die "updateGalleryDB Invalid argument $filename $title";
  }
  
  my $sth;

  $title =$summary=$description =  encode('UTF-8',$title);#g_description
  $keywords = "";#g_keywords
  $ownerId = $CONTENT_USER_ID;#6087;#g_ownerId
  $originationTimestamp = time();#g_originationTimestamp

  #画像DBを更新する
  eval{
    $sth = $db->prepare("INSERT INTO g2_Item VALUES (?,0, ?, ?, ?, ?, ?, ?,?)");
    $sth->execute($id,$description,$keywords,$ownerId,$summary,$title,0,$originationTimestamp);
    $sth->finish;
    
    $sth = $db->prepare("INSERT INTO g2_FileSystemEntity VALUES(?,?)");
    $sth->execute($id,$filename);
    $sth->finish;
    
    $sth = $db->prepare("INSERT INTO g2_Entity (g_id,g_creationTimestamp,g_isLinkable,g_modificationTimestamp,g_serialNumber,g_entityType) VALUES (?,?,1,?,1,\"GalleryPhotoItem\")");
    $sth->execute($id,$originationTimestamp,$originationTimestamp);
    $sth->finish;
    
    $sth = $db->prepare('INSERT INTO g2_PhotoItem VALUES(?,0,0)');
    $sth->execute($id);
    $sth->finish;
    
    $sth = $db->prepare('INSERT INTO g2_ChildEntity VALUES(?,?)');
    $sth->execute($id,$CONTENT_ALBUM_ID);
    $sth->finish;
    
    $sth = $db->prepare('INSERT INTO g2_DataItem VALUES(?,"image/jpeg",?)');
    $sth->execute($id,$fileLength);
    $sth->finish;
    
    $sth = $db->prepare("INSERT INTO g2_ItemAttributesMap VALUES(?,0,1000,?)");
    $sth->execute($id,$CONTENT_ALBUM_ID."/");
    $sth->finish;
    
    $sth = $db->prepare("INSERT INTO g2_AccessSubscriberMap VALUES(?,?)");
    $sth->execute($id,$CONTENT_ACCESSLIST_ID);
    $sth->finish;
    
    $sth = $db->prepare("UPDATE g2_SequenceId set id=?");
    $sth->execute($id+1);
    $sth->finish;
  };
  if($@){
    die "#error: can't update gallery DB: id=$id";
  }
  return;
}

#記事の先頭に画像へのリンクをつける
sub updateContentDB{
  my $db = shift;
  my $contentId = shift;
  my $fulltext = shift;

  #引数のチェック
  if(!defined($db) or !defined($contentId) or !defined($fulltext)){
    die "#exception: updateContentDB() Invalid argument $db ,$contentId , $fulltext";
  }
  

  #記事本文を更新
  my $sth = $db->prepare("UPDATE stym_content SET `fulltext` = ? WHERE id = ?");
  $sth->execute($fulltext,$contentId) or die"update content error id=$contentId";
  $sth->finish;
}


#エラーを出力する
sub writeError{
  my $error = shift;
  my $header = shift;
  open(FILE, ">> $LOG_FILE") or die "log file can't open";
  print FILE "to:$header{'to'} ,from:$header{'from'}";
  print FILE $error;
  close(FILE);
}

#データベースに接続する
sub connect{
  my $databaseName = shift;
  my $db;

  eval{
    $db = DBI->connect("DBI:mysql:$databaseName:localhost", $DATABASE_ID, $DATABASE_PASS);
    my $sth = $db->prepare("set names utf8");
    $sth->execute();
    $sth->finish;
  };
  if($@){
    die $@;
  }
  
  return $db;
}
sub prepareDB{
  my $db = &connect(shift) or die;
  #トランザクションのための設定
  $db->{AutoCommit}=0;
  $db->{RaiseError}=1;

  return $db;
}

#データベースから画像のIDを取得する
sub getImageId{
  my $db = shift;
  #idを決める
  my $sth = $db->prepare("SELECT id FROM g2_SequenceId");
  $sth->execute;
  my $id = ($sth->fetchrow_array)[0]+2;#g_id
  $sth->finish;
  return $id;
}

#fulltextを取得し、imgタグを付加する
sub createContentText{
  my $db = shift;
  my $contentId=shift;
  my $imageId = shift;

  my $sth;
  #引数のチェック
  if(!defined($db) or !defined($contentId) or !defined($imageId)){
    die "#exception: updateContent() Invalid argument $contentId , $imageId";
  }

  #現在の記事本文を取得
  $sth = $db->prepare("SELECT `fulltext` FROM stym_content WHERE id = ?");
  $sth->execute($contentId);
  my @array = ($sth->fetchrow_array) or die "#error: updateContent() Invalid contentId = $contentId";
  my $fulltext = $array[0];#g_id
  $sth->finish;

  if($fulltext =~ /<img\ssrc.*>/i){
    #すでにimgタグが付いている
    die "warning: double image tag id=$contentId";
  }
  
  my $imageTag= "<img src=\"$GALLERY_LOCATION/main.php\?g2_view=core.DownloadItem&g2_itemId=".$imageId.'&g2_serialNumber=1">';
  return $imageTag.$fulltext;
}
